@@ -8,6 +8,8 @@
 @var TURTLE: to be used to set the return format to Turtle
 @var N3: to be used to set the return format to N3 (for most of the SPARQL services this is equivalent to Turtle)
 @var RDF: to be used to set the return RDF Graph
+@var CSV: to be used to set the return format to CSV
+@var TSV: to be used to set the return format to TSV
 
 @var POST: to be used to set HTTP POST
 @var GET: to be used to set HTTP GET. This is the default.
@@ -38,38 +40,89 @@ from .KeyCaseInsensitiveDict import KeyCaseInsensitiveDict
 from .SPARQLExceptions import QueryBadFormed, EndPointNotFound, EndPointInternalError
 from SPARQLWrapper import __agent__
 
-#  Possible output format keys...
+#  From <https://www.w3.org/TR/sparql11-protocol/#query-success>
+#  The response body of a successful query operation with a 2XX response is either:
+#  * SELECT and ASK: a SPARQL Results Document in XML, JSON, or CSV/TSV format.
+#  * DESCRIBE and CONSTRUCT: an RDF graph serialized, for example, in the RDF/XML syntax, or an equivalent RDF graph serialization.
+#
+#  Possible parameter keys and values...
 #  Examples:
 #  - ClioPatria: the SWI-Prolog Semantic Web Server <http://cliopatria.swi-prolog.org/home>
-#    * Parameter "format" must be one of "rdf+xml", "json", "csv", "application/sparql-results+xml" or "application/sparql-results+json".
+#    * Parameter key: "format"
+#    * Parameter value must have one of these values: "rdf+xml", "json", "csv", "application/sparql-results+xml" or "application/sparql-results+json".
+#
 #  - OpenLink Virtuoso  <http://virtuoso.openlinksw.com>
-#    * Multiple values, like directly:
+#    * Parameter key: "format" or "output"
+#    * Parameter value, like directly:
 #      "text/html" (HTML), "text/x-html+tr" (HTML (Faceted Browsing Links)), "application/vnd.ms-excel"
 #      "application/sparql-results+xml" (XML), "application/sparql-results+json", (JSON)
 #      "application/javascript" (Javascript), "text/turtle" (Turtle), "application/rdf+xml" (RDF/XML)
 #      "text/plain" (N-Triples), "text/csv" (CSV), "text/tab-separated-values" (TSV)
-#    * Multiple values, like indirectly:
-#      "HTML", "JSON", "XML", "TURTLE"
+#    * Parameter value, like indirectly:
+#      "HTML" (alias text/html), "JSON" (alias application/sparql-results+json), "XML" (alias application/sparql-results+xml), "TURTLE" (alias text/rdf+n3), JavaScript (alias application/javascript)
 #       See  <http://virtuoso.openlinksw.com/dataspace/doc/dav/wiki/Main/VOSSparqlProtocol#Additional HTTP Response Formats -- SELECT>
+#
 #  - Fuseki (formerly there was Joseki) <https://jena.apache.org/documentation/serving_data/>
+#    * Parameter key: "format" or "output"
+#      See Fuseki 1: https://github.com/apache/jena/blob/master/jena-fuseki1/src/main/java/org/apache/jena/fuseki/HttpNames.java
+#      See Fuseki 2: https://github.com/apache/jena/blob/master/jena-arq/src/main/java/org/apache/jena/riot/web/HttpNames.java
 #    * Fuseki 1 - Short names for "output=" : "json", "xml", "sparql", "text", "csv", "tsv", "thrift"
 #      See <https://github.com/apache/jena/blob/master/jena-fuseki1/src/main/java/org/apache/jena/fuseki/servlets/ResponseResultSet.java>
 #    * Fuseki 2 - Short names for "output=" : "json", "xml", "sparql", "text", "csv", "tsv", "thrift"
 #      See <https://github.com/apache/jena/blob/master/jena-fuseki2/jena-fuseki-core/src/main/java/org/apache/jena/fuseki/servlets/ResponseResultSet.java>
+#
 #  - Eclipse RDF4J (formerly known as Sesame) <http://rdf4j.org/>
-#    * Uses only content negotiation. See <http://rdf4j.org/doc/the-rdf4j-server-rest-api/#The_QUERY_operation>
+#    * Uses only content negotiation (no URL parameters).
+#    * See <http://rdf4j.org/doc/the-rdf4j-server-rest-api/#The_QUERY_operation>
+#
 #  - RASQAL <http://librdf.org/rasqal/>
+#    * Parameter key: "results"
 #    * Uses roqet as RDF query utility
-#      For variable bindings, the values of FORMAT vary upon what Rasqal supports but include simple 
-#      for a simple text format (default), xml for the SPARQL Query Results XML format, csv for SPARQL CSV, 
+#      For variable bindings, the values of FORMAT vary upon what Rasqal supports but include simple
+#      for a simple text format (default), xml for the SPARQL Query Results XML format, csv for SPARQL CSV,
 #      tsv for SPARQL TSV, rdfxml and turtle for RDF syntax formats, and json for a JSON version of the results.
 #
-#      For RDF graph results, the values of FORMAT are ntriples (N-Triples, default), 
-#      rdfxml-abbrev (RDF/XML Abbreviated), rdfxml (RDF/XML), turtle (Turtle), 
-#      json (RDF/JSON resource centric), json-triples (RDF/JSON triples) or 
+#      For RDF graph results, the values of FORMAT are ntriples (N-Triples, default),
+#      rdfxml-abbrev (RDF/XML Abbreviated), rdfxml (RDF/XML), turtle (Turtle),
+#      json (RDF/JSON resource centric), json-triples (RDF/JSON triples) or
 #      rss-1.0 (RSS 1.0, also an RDF/XML syntax).
 #
 #      See <http://librdf.org/rasqal/roqet.html>
+#
+#  - Marklogic <http://marklogic.com>
+#    * Uses content negotiation (no URL parameters).
+#    * You can use following methods to query triples <https://docs.marklogic.com/guide/semantics/semantic-searches#chapter>:
+#      - SPARQL mode in Query Console. For details, see Querying Triples with SPARQL
+#      - XQuery using the semantics functions, and Search API, or a combination of XQuery and SPARQL. For details, see Querying Triples with XQuery or JavaScript.
+#      - HTTP via a SPARQL endpoint. For details, see Using Semantics with the REST Client API.
+#    * Formats are specified as part of the HTTP Accept headers of the REST request. <https://docs.marklogic.com/guide/semantics/REST#id_92428>
+#      - When you query the SPARQL endpoint with REST Client APIs, you can specify the result output format.  <https://docs.marklogic.com/guide/semantics/REST#id_54258>
+#        The response type format depends on the type of query and the MIME type in the HTTP Accept header.
+#      - This table describes the MIME types and Accept Header/Output formats (MIME type) for different types of SPARQL queries. See <https://docs.marklogic.com/guide/semantics/REST#id_54258> and <https://docs.marklogic.com/guide/semantics/loading#id_70682>
+#        SELECT "application/sparql-results+xml", "application/sparql-results+json", "text/html", "text/csv"
+#        CONSTRUCT or DESCRIBE "application/n-triples", "application/rdf+json", "application/rdf+xml", "text/turtle", "text/n3", "application/n-quads", "application/trig"
+#
+#  - AllegroGraph <https://franz.com/agraph/allegrograph/>
+#    * Uses only content negotiation (no URL parameters).
+#    * The server always looks at the Accept header of a request, and tries to
+#      generate a response in the format that the client asks for. If this fails,
+#      a 406 response is returned. When no Accept, or an Accept of */* is specified,
+#      the server prefers text/plain, in order to make it easy to explore the interface from a web browser.
+#    * Accept header expected (values returned by server when a wrong header is sent):
+#    ** SELECT
+#    *** application/sparql-results+xml
+#    *** application/sparql-results+json (and application/json)
+#    *** text/csv
+#    *** text/tab-separated-values
+#    *** OTHERS: application/sparql-results+ttl, text/integer, application/x-lisp-structured-expression, text/table, application/processed-csv, text/simple-csv, application/x-direct-upis
+#
+#    ** CONSTRUCT
+#    *** application/rdf+xml
+#    *** text/rdf+n3
+#    *** OTHERS: text/integer, application/json, text/plain, text/x-nquads, application/trix, text/table, application/x-direct-upis
+#
+#      See <https://franz.com/agraph/support/documentation/current/http-protocol.html>
+
 
 JSON   = "json"
 JSONLD = "json-ld"
@@ -78,7 +131,9 @@ TURTLE = "turtle"
 N3     = "n3"
 RDF    = "rdf"
 RDFXML = "rdf+xml"
-_allowedFormats = [JSON, XML, TURTLE, N3, RDF, RDFXML]
+CSV    = "csv"
+TSV    = "tsv"
+_allowedFormats = [JSON, XML, TURTLE, N3, RDF, RDFXML, CSV, TSV]
 
 # Possible HTTP methods
 POST = "POST"
@@ -90,7 +145,7 @@ BASIC = "BASIC"
 DIGEST = "DIGEST"
 _allowedAuth = [BASIC, DIGEST]
 
-# Possible SPARQL/SPARUL query type
+# Possible SPARQL/SPARUL query type (aka SPARQL Query forms)
 SELECT     = "SELECT"
 CONSTRUCT  = "CONSTRUCT"
 ASK        = "ASK"
@@ -126,9 +181,11 @@ _SPARQL_JSON     = ["application/sparql-results+json", "text/javascript", "appli
 _RDF_XML         = ["application/rdf+xml"]
 _RDF_N3          = ["text/rdf+n3", "application/n-triples", "application/turtle", "application/n3", "text/n3", "text/turtle"]
 _RDF_JSONLD      = ["application/x-json+ld", "application/ld+json"]
+_CSV             = ["text/csv"]
+_TSV             = ["text/tab-separated-values"]
+_XML             = ["application/xml"]
 _ALL             = ["*/*"]
-_RDF_POSSIBLE    = _RDF_XML + _RDF_N3
-_SPARQL_POSSIBLE = _SPARQL_XML + _SPARQL_JSON + _RDF_XML + _RDF_N3
+_RDF_POSSIBLE    = _RDF_XML + _RDF_N3 + _XML
 _SPARQL_PARAMS   = ["query"]
 
 try:
@@ -139,11 +196,11 @@ except ImportError:
     #warnings.warn("JSON-LD disabled because no suitable support has been found", RuntimeWarning)
     pass
 
-# This is very ugly. The fact is that the key for the choice of the output format is not defined. 
-# Virtuoso uses 'format', joseki uses 'output', rasqual seems to use "results", etc. Lee Feigenbaum 
-# told me that virtuoso also understand 'output' these days, so I removed 'format'. I do not have 
-# info about the others yet, ie, for the time being I keep the general mechanism. Hopefully, in a 
-# future release, I can get rid of that. However, these processors are (hopefully) oblivious to the 
+# This is very ugly. The fact is that the key for the choice of the output format is not defined.
+# Virtuoso uses 'format', joseki uses 'output', rasqual seems to use "results", etc. Lee Feigenbaum
+# told me that virtuoso also understand 'output' these days, so I removed 'format'. I do not have
+# info about the others yet, ie, for the time being I keep the general mechanism. Hopefully, in a
+# future release, I can get rid of that. However, these processors are (hopefully) oblivious to the
 # parameters they do not understand. So: just repeat all possibilities in the final URI. UGLY!!!!!!!
 _returnFormatSetting = ["format", "output", "results"]
 
@@ -183,7 +240,7 @@ class SPARQLWrapper(object):
         is up to the endpoint to react or not, this wrapper does not check.
 
         Possible values:
-        L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDFXML} (constants in this module). The value can also be set via explicit
+        L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). The value can also be set via explicit
         call, see below.
         @type returnFormat: string
         @keyword defaultGraph: URI for the default graph. Default is None, the value can be set either via an L{explicit call<addDefaultGraph>} or as part of the query string.
@@ -196,6 +253,7 @@ class SPARQLWrapper(object):
         self.passwd = None
         self.http_auth = BASIC
         self._defaultGraph = defaultGraph
+        self.onlyConneg = False # Only Content Negotiation
 
         if returnFormat in _allowedFormats:
             self._defaultReturnFormat = returnFormat
@@ -206,7 +264,8 @@ class SPARQLWrapper(object):
 
     def resetQuery(self):
         """Reset the query, ie, return format, query, default or named graph settings, etc,
-        are reset to their default values."""
+        are reset to their default values.
+        """
         self.parameters = {}
         if self._defaultGraph:
             self.addParameter("default-graph-uri", self._defaultGraph)
@@ -219,7 +278,7 @@ class SPARQLWrapper(object):
     def setReturnFormat(self, format):
         """Set the return format. If not an allowed value, the setting is ignored.
 
-        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF} (constants in this module). All other cases are ignored.
+        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). All other cases are ignored.
         @type format: str
         """
         if format in _allowedFormats :
@@ -227,12 +286,12 @@ class SPARQLWrapper(object):
         elif format == JSONLD:
             raise ValueError("Current instance does not support JSON-LD; you might want to install the rdflib-json package.")
         else:
-            raise ValueError("Invalid format '%s'; current instance supports: %s.", (format, ", ".join(_allowedFormats)))
+            warnings.warn("Ignore format '%s'; current instance supports: %s." %(format, ", ".join(_allowedFormats)), SyntaxWarning)
 
     def supportsReturnFormat(self, format):
         """Check if a return format is supported.
 
-        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF} (constants in this module). All other cases are ignored.
+        @param format: Possible values: are L{JSON}, L{XML}, L{TURTLE}, L{N3}, L{RDF}, L{RDFXML}, L{CSV}, L{TSV} (constants in this module). All other cases are ignored.
         @type format: bool
         """
         return (format in _allowedFormats)
@@ -245,9 +304,17 @@ class SPARQLWrapper(object):
         """
         self.timeout = int(timeout)
 
+    def setOnlyConneg(self, onlyConneg):
+        """Set this option for allowing (or not) only HTTP Content Negotiation (so dismiss the use of HTTP parameters).
+
+        @param onlyConneg: True if only HTTP Content Negotiation is allowed; False is HTTP parameters are allowed also.
+        @type onlyConneg: bool
+        """
+        self.onlyConneg = onlyConneg
+
     def setRequestMethod(self, method):
         """Set the internal method to use to perform the request for query or
-        update operations, either URL-encoded (C{SPARQLWrapper.URLENCODED}) or 
+        update operations, either URL-encoded (C{SPARQLWrapper.URLENCODED}) or
         POST directly (C{SPARQLWrapper.POSTDIRECTLY}).
         Further details at U{http://www.w3.org/TR/sparql11-protocol/#query-operation}
         and U{http://www.w3.org/TR/sparql11-protocol/#update-operation}.
@@ -281,7 +348,7 @@ class SPARQLWrapper(object):
     def addExtraURITag(self, key, value):
         """
             Some SPARQL endpoints require extra key value pairs.
-            E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing 
+            E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing
             virtuoso to retrieve graphs that are not stored in its local database.
             @param key: key of the query part
             @type key: string
@@ -294,7 +361,7 @@ class SPARQLWrapper(object):
     def addCustomParameter(self, name, value):
         """
             Method is kept for backwards compatibility. Historically, it "replaces" parameters instead of adding
-            @param name: name 
+            @param name: name
             @type name: string
             @param value: value
             @type value: string
@@ -307,9 +374,9 @@ class SPARQLWrapper(object):
     def addParameter(self, name, value):
         """
             Some SPARQL endpoints allow extra key value pairs.
-            E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing 
+            E.g., in virtuoso, one would add C{should-sponge=soft} to the query forcing
             virtuoso to retrieve graphs that are not stored in its local database.
-            @param name: name 
+            @param name: name
             @type name: string
             @param value: value
             @type value: string
@@ -326,7 +393,7 @@ class SPARQLWrapper(object):
     def clearParameter(self, name):
         """
             Clear the values ofd a concrete parameter.
-            @param name: name 
+            @param name: name
             @type name: string
             @rtype: bool
         """
@@ -363,11 +430,11 @@ class SPARQLWrapper(object):
         else:
             valid_types = ", ".join(_allowedAuth)
             raise ValueError("Value should be one of {0}".format(valid_types))
-                    
+
     def setQuery(self, query):
         """
-            Set the SPARQL query text. Note: no check is done on the validity of the query 
-            (syntax or otherwise) by this module, except for testing the query type (SELECT, 
+            Set the SPARQL query text. Note: no check is done on the validity of the query
+            (syntax or otherwise) by this module, except for testing the query type (SELECT,
             ASK, etc). Syntax and validity checking is done by the SPARQL service itself.
             @param query: query text
             @type query: string
@@ -396,7 +463,7 @@ class SPARQLWrapper(object):
             Parse the SPARQL query and return its type (ie, L{SELECT}, L{ASK}, etc).
 
             Note that the method returns L{SELECT} if nothing is specified. This is just to get all other
-            methods running; in fact, this means that the query is erronous, because the query must be,
+            methods running; in fact, this means that the query is erroneous, because the query must be,
             according to the SPARQL specification, one of Select, Ask, Describe, or Construct. The
             SPARQL endpoint should raise an exception (via urllib) for such syntax error.
 
@@ -411,7 +478,7 @@ class SPARQLWrapper(object):
         except AttributeError:
             warnings.warn("not detected query type for query '%s'" % query.replace("\n", " "), RuntimeWarning)
             r_queryType = None
-    
+
         if r_queryType in _allowedQueryTypes :
             return r_queryType
         else :
@@ -431,6 +498,11 @@ class SPARQLWrapper(object):
         """
         try:
             from keepalive import HTTPHandler
+
+            if urllib2._opener and any(isinstance(h, HTTPHandler) for h in urllib2._opener.handlers):
+                # already installed
+                return
+
             keepalive_handler = HTTPHandler()
             opener = urllib.request.build_opener(keepalive_handler)
             urllib.request.install_opener(opener)
@@ -450,7 +522,7 @@ class SPARQLWrapper(object):
         return not self.isSparqlUpdateRequest()
 
     def _cleanComments(self, query):
-        # remove all occurance singleline comments (issues #32 and #77)
+        # remove all occurrence singleline comments (issues #32 and #77)
         return re.sub(self.comments_pattern, "\n\n" , query)
 
     def _getRequestEncodedParameters(self, query=None):
@@ -464,8 +536,17 @@ class SPARQLWrapper(object):
         # Virtuoso uses 'format',sparqler uses 'output'
         # However, these processors are (hopefully) oblivious to the parameters they do not understand.
         # So: just repeat all possibilities in the final URI. UGLY!!!!!!!
-        for f in _returnFormatSetting:
-            query_parameters[f] = [self.returnFormat]
+        if not self.onlyConneg:
+            for f in _returnFormatSetting:
+                query_parameters[f] = [self.returnFormat]
+                # Virtuoso is not supporting a correct Accept header and an unexpected "output"/"format" parameter value. It returns a 406.
+                # "tsv" and "json-ld" are not supported as a correct "output"/"format" parameter value but "text/tab-separated-values" is a valid value,
+                # and there is no problem to send both.
+                if self.returnFormat in [TSV, JSONLD]:
+                    acceptHeader = self._getAcceptHeader() # to obtain the mime-type "text/tab-separated-values"
+                    if "*/*" in acceptHeader:
+                        acceptHeader="" # clear the value in case of "*/*"
+                    query_parameters[f]+= [acceptHeader]
 
         pairs = (
             "%s=%s" % (
@@ -483,11 +564,16 @@ class SPARQLWrapper(object):
                 acceptHeader = ",".join(_SPARQL_XML)
             elif self.returnFormat == JSON:
                 acceptHeader = ",".join(_SPARQL_JSON)
+            elif self.returnFormat == CSV: # Allowed for SELECT and ASK (https://www.w3.org/TR/2013/REC-sparql11-protocol-20130321/#query-success) but only described for SELECT (https://www.w3.org/TR/sparql11-results-csv-tsv/)
+                acceptHeader = ",".join(_CSV)
+            elif self.returnFormat == TSV: # Allowed for SELECT and ASK (https://www.w3.org/TR/2013/REC-sparql11-protocol-20130321/#query-success) but only described for SELECT (https://www.w3.org/TR/sparql11-results-csv-tsv/)
+                acceptHeader = ",".join(_TSV)
             else:
                 acceptHeader = ",".join(_ALL)
+                warnings.warn("Sending Accept header '*/*' because unexpected returned format '%s' in a '%s' SPARQL query form" % (self.returnFormat, self.queryType), RuntimeWarning)
         elif self.queryType in [INSERT, DELETE]:
             acceptHeader = "*/*"
-        else:
+        else: #CONSTRUCT, DESCRIBE
             if self.returnFormat == N3 or self.returnFormat == TURTLE:
                 acceptHeader = ",".join(_RDF_N3)
             elif self.returnFormat == XML:
@@ -496,12 +582,13 @@ class SPARQLWrapper(object):
                 acceptHeader = ",".join(_RDF_JSONLD)
             else:
                 acceptHeader = ",".join(_ALL)
+                warnings.warn("Sending Accept header '*/*' because unexpected returned format '%s' in a '%s' SPARQL query form" % (self.returnFormat, self.queryType), RuntimeWarning)
         return acceptHeader
 
     def _createRequest(self):
         """Internal method to create request according a HTTP method. Returns a
         C{urllib2.Request} object of the urllib2 Python library
-        @return: request
+        @return: request a C{urllib2.Request} object of the urllib2 Python library
         """
         request = None
 
@@ -564,7 +651,6 @@ class SPARQLWrapper(object):
         """
         if self.timeout:
             socket.setdefaulttimeout(self.timeout)
-
         request = self._createRequest()
 
         try:
@@ -696,9 +782,9 @@ class QueryResult(object):
         except ImportError:
             from rdflib import ConjunctiveGraph
         retval = ConjunctiveGraph()
-        # this is a strange hack. If the publicID is not set, rdflib (or the underlying xml parser) makes a funny
-        #(and, as far as I could see, meaningless) error message...
-        retval.load(self.response, publicID=' ')
+        # (DEPRECATED) this is a strange hack. If the publicID is not set, rdflib (or the underlying xml parser) makes a funny
+        # (DEPRECATED) (and, as far as I could see, meaningless) error message...
+        retval.load(self.response) # (DEPRECATED) publicID=' ')
         return retval
 
     def _convertN3(self):
@@ -710,6 +796,24 @@ class QueryResult(object):
         """
         return self.response.read()
 
+    def _convertCSV(self):
+        """
+        Convert a CSV result into a string. This method can be overwritten in a subclass
+        for a different conversion method.
+        @return: converted result
+        @rtype: string
+        """
+        return self.response.read()
+
+    def _convertTSV(self):
+        """
+        Convert a TSV result into a string. This method can be overwritten in a subclass
+        for a different conversion method.
+        @return: converted result
+        @rtype: string
+        """
+        return self.response.read()
+
     def _convertJSONLD(self):
         """
         Convert a RDF JSON-LDresult into an RDFLib triple store. This method can be overwritten
@@ -719,7 +823,7 @@ class QueryResult(object):
         """
         from rdflib import ConjunctiveGraph
         retval = ConjunctiveGraph()
-        retval.load(self.response, format='json-ld', publicID=' ')
+        retval.load(self.response, format='json-ld')# (DEPRECATED), publicID=' ')
         return retval
 
     def convert(self):
@@ -727,7 +831,9 @@ class QueryResult(object):
         Encode the return value depending on the return format:
             - in the case of XML, a DOM top element is returned;
             - in the case of JSON, a simplejson conversion will return a dictionary;
-            - in the case of RDF/XML, the value is converted via RDFLib into a Graph instance.
+            - in the case of RDF/XML, the value is converted via RDFLib into a Graph instance;
+            - in the case of RDF Turtle/N3, a string is returned;
+            - in the case of CSV/TSV, a string is returned.
         In all other cases the input simply returned.
 
         @return: the converted query result. See the conversion methods for more details.
@@ -740,12 +846,16 @@ class QueryResult(object):
                 message = "Format requested was %s, but %s (%s) has been returned by the endpoint"
                 warnings.warn(message % (requested.upper(), format_name, mime), RuntimeWarning)
 
+        # TODO. In order to compare properly, the requested QueryType (SPARQL Query Form) is needed. For instance, the unexpected N3 requested for a SELECT would return XML
         if "content-type" in self.info():
-            ct = self.info()["content-type"]
+            ct = self.info()["content-type"] # returned Content-Type value
 
             if _content_type_in_list(ct, _SPARQL_XML):
                 _validate_format("XML", [XML], ct, self.requestedFormat)
                 return self._convertXML()
+            elif _content_type_in_list(ct, _XML):
+                _validate_format("XML", [XML], ct, self.requestedFormat)
+                return self._convertXML()
             elif _content_type_in_list(ct, _SPARQL_JSON):
                 _validate_format("JSON", [JSON], ct, self.requestedFormat)
                 return self._convertJSON()
@@ -755,11 +865,17 @@ class QueryResult(object):
             elif _content_type_in_list(ct, _RDF_N3):
                 _validate_format("N3", [N3, TURTLE], ct, self.requestedFormat)
                 return self._convertN3()
+            elif _content_type_in_list(ct, _CSV):
+                _validate_format("CSV", [CSV], ct, self.requestedFormat)
+                return self._convertCSV()
+            elif _content_type_in_list(ct, _TSV):
+                _validate_format("TSV", [TSV], ct, self.requestedFormat)
+                return self._convertTSV()
             elif _content_type_in_list(ct, _RDF_JSONLD):
                 _validate_format("JSON(-LD)", [JSONLD, JSON], ct, self.requestedFormat)
                 return self._convertJSONLD()
 
-        warnings.warn("unknown response content type, returning raw response...", RuntimeWarning)
+        warnings.warn("unknown response content type '%s' returning raw response..." %(ct), RuntimeWarning)
         return self.response.read()
 
     def print_results(self, minWidth=None):
@@ -791,7 +907,7 @@ class QueryResult(object):
             for var in results["head"]["vars"] :
                 result = self.__get_prettyprint_string_sparql_var_result(result[var])
                 width[index] = max(width[index], len(result))
-                index =+ 1
+                index += 1
         return width
 
     def __get_prettyprint_string_sparql_var_result(self, result):
